#!/bin/bash

# This script looks for the necessary BOOST sub-libraries.
# If found, it prints out the full path and returns with exit code 0.
# If not, it prints out a warning and returns with exit code 1.

SUBLIBS="thread system filesystem"

##################################################################
if [ "$#" != "1" ]
then
  echo "$0: ERROR: requires 1 arg (full path of BOOST_HDR_DIR)"
  exit 1
fi

BOOST_HDR_DIR="$1"

# Special handling if BOOST is not fully installed...
if [ -d "$BOOST_HDR_DIR/stage" -a -r "$BOOST_HDR_DIR/stage" ]
then
  BOOST_LIB_DIR="$BOOST_HDR_DIR/stage/lib"
else
  BOOST_LIB_DIR="`dirname $BOOST_HDR_DIR`/lib"
fi

MISSING_SUBLIBS=
for sublib in $SUBLIBS
do
  if [ \! -f "$BOOST_LIB_DIR/libboost_$sublib.a" ]
  then
    MISSING_SUBLIBS="$MISSING_SUBLIBS $sublib"
  fi
done
    

if [ \! -z "$MISSING_SUBLIBS" ]
then
  echo "WARNING: CoCoA-5 needs the following BOOST sublibraries: $MISSING_SUBLIBS"
  exit 1
fi

echo "$BOOST_LIB_DIR"
exit 0
