//   Copyright (c)  2006,2009  John Abbott

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


// These are from directory TmpFactorDir/
#include "mpz_alias.h"
#include "primes.h"
#include "FF.h"
#include "DUPFF.h"
#include "DUPFFfactor.h"
//#include "DUPFFlist.h"
#include "WGD.h"

// These are from directory TmpFactorDir/multivariate/
#include "DMPZgcd.h"
#include "DMPZfactor.h"
#include "DMPZfactor_modp.h"

/***************************************************************************/
/* CoCoALib includes */

#include "CoCoA/TmpFactor.H"

#include "CoCoA/IntOperations.H"
#include "CoCoA/CanonicalHom.H"
#include "CoCoA/FractionField.H"
#include "CoCoA/NumTheory.H"
#include "CoCoA/PolyRing.H"
#include "CoCoA/QuotientRing.H"
#include "CoCoA/RingQQ.H"
#include "CoCoA/RingZZ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/convert.H"
#include "CoCoA/utils.H"

//#include <iostream>
#include <cstdlib>
using std::malloc;// someone actually uses malloc!!!!
#include <vector>
using std::vector;

/***************************************************************************/

namespace CoCoA
{


  DUPFF ConvertToDUPFF(ConstRefRingElem f)
  {
    CoCoA_ASSERT(IsSparsePolyRing(owner(f)));
    SparsePolyRing Rx = AsSparsePolyRing(owner(f));
    ring R = CoeffRing(Rx);
    CoCoA_ASSERT(IsField(R) && IsQuotientRing(R) && IsZZ(BaseRing(AsQuotientRing(R))));
    const BigInt P = characteristic(R);
    long p = ConvertTo<long>(P);
    CoCoA_ASSERT(p < 32768); /// *** BUG BUG BUG ***

    FF Fp = FFctor(p);
    FFselect(Fp);
    const long d = StdDeg(f);
    DUPFF ans = DUPFFnew(d);
    { FFelem* CoeffVec = ans->coeffs; for (long i=0; i<=d; ++i) CoeffVec[i] = 0; }
    for (SparsePolyIter it = BeginIter(f); !IsEnded(it); ++it)
    {
      BigInt c;
      if (!IsInteger(c, coeff(it))) CoCoA_ERROR("coeff not in Z","ConvertToDUPFF");
      long coeff = ConvertTo<long>(c);
      if (coeff < 0) coeff += p;
      long exp = StdDeg(PP(it));
      ans->coeffs[exp] = coeff;
    }
    ans->deg = d;
///???    FFdtor(Fp);
    return ans;
  }


  RingElem ConvertDUPFFToRingElem(DUPFF F, ConstRefRingElem x)
  {
    SparsePolyRing P = AsSparsePolyRing(owner(x));
    PPMonoidElem t = LPP(x);
    RingElem ans(P);
    const int D = DUPFFdeg(F);
    for (int i=D; i >= 0; --i)
    {
      if (F->coeffs[i] == 0) continue;
      ans += monomial(P, F->coeffs[i], power(t,i));
    }
    return ans;
  }

/////?????  DUPZ ConvertToDUPZ(ConstRefRingElem f) { ... }

// Exception safe???  You're kidding, right?
  DMPZ ConvertToDMPZ(ConstRefRingElem f)
  {
    CoCoA_ASSERT(IsSparsePolyRing(owner(f)));
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    ring R = CoeffRing(P);
    CoCoA_ASSERT(characteristic(R) == 0);
    CoCoA_ASSERT(IsZZ(R) || IsFractionFieldOfGCDDomain(R));

    const size_t nvars = NumIndets(P);
    DMPZ g = NULL;
    std::vector<long> expv(nvars);
    for (SparsePolyIter it = BeginIter(f); !IsEnded(it); ++it)
    {
      BigInt c;
      if (!IsInteger(c, coeff(it))) CoCoA_ERROR("coeff not in Z","factorize");
      exponents(expv, PP(it));
      int *exps = (int*)malloc(nvars*sizeof(int)); // block absorbed by g
      std::copy(expv.begin(), expv.end(), exps);
      g = DMPZprepend(mpzref(c), exps, g);
    }

    return DMPZreverse(g); /* restore original order of the terms */
  }


  void ConvertDMPZToRingElem(RefRingElem f, DMPZ poly)
  {
    CoCoA_ASSERT(IsSparsePolyRing(owner(f)));
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    PPMonoid M = PPM(P);
    PPMonoidElem t(M);
    f = 0;
    const size_t nvars = NumIndets(P);
    std::vector<long> exps(nvars);
    for (DMPZ iter = poly; iter; iter = iter->next)
    {
      if (mpz_sgn(iter->coeff) == 0) continue;
      const BigInt c(iter->coeff);
      std::copy(iter->exps, iter->exps+nvars, exps.begin());
      f += monomial(P, c, PPMonoidElem(M, exps));
    }
  }

  factorization<RingElem> ConvertToFactorList(DMPZfactors facpows, ring Rx)
  {
    RingElem content = RingElem(Rx, BigInt(facpows->content));
    RingElem factor(Rx);
    vector<RingElem> facs;
    vector<long> exps;

    DMPZlist iter;
    int i;
    for (i=1, iter=facpows->list; iter; ++i, iter = iter->next)
    {
      ConvertDMPZToRingElem(factor, iter->poly);
      facs.push_back(factor);
      exps.push_back(iter->deg);
    }
    DMPZfactors_dtor(facpows);  // NASTY - we destroy one of our args!!
    return factorization<RingElem>(facs, exps, content);
  }


  RingElem GCD_DUPFF(ConstRefRingElem f, ConstRefRingElem g)
  {
    CoCoA_ASSERT(owner(f) == owner(g));
    CoCoA_ASSERT(IsSparsePolyRing(owner(f)));
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    vector<long> index = indets(f);
    RingElem x = indet(P, index[0]);
    DUPFF F = ConvertToDUPFF(f);
    DUPFF G = ConvertToDUPFF(g);
    DUPFFgcd2(F,G);
    RingElem ans = ConvertDUPFFToRingElem(F, x);
    DUPFFfree(G);
    DUPFFfree(F);
    return ans;
  }


  RingElem GCD_DMPZ(ConstRefRingElem f, ConstRefRingElem g)
  {
    CoCoA_ASSERT(owner(f) == owner(g));
    CoCoA_ASSERT(IsSparsePolyRing(owner(f)));
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    CoCoA_ASSERT(IsZZ(CoeffRing(P)) || IsQQ(CoeffRing(P)));

    if (IsQQ(CoeffRing(P)))
    {
      // case P = QQ[x,y,z,...]
      RingHom coeff = CoeffEmbeddingHom(P);
      RingElem ContF = content(f);
      RingElem ContG = content(g);
      NVARS = NumIndets(P); // BUG BUG BUG:  NVARS is a global!!!
      DMPZ F = ConvertToDMPZ(f/coeff(ContF));
      DMPZ G = ConvertToDMPZ(g/coeff(ContG));
      DMPZ H = DMPZgcd(F,G);
      DMPZdtor(G);
      DMPZdtor(F);

      RingElem ans(P);
      ConvertDMPZToRingElem(ans, H);
      return ans;
    }
    if (IsZZ(CoeffRing(P)))
    {
      // case P = ZZ[x,y,z,...]
      NVARS = NumIndets(P); // BUG BUG BUG:  NVARS is a global!!!
      DMPZ F = ConvertToDMPZ(f);
      DMPZ G = ConvertToDMPZ(g);
      DMPZ H = DMPZgcd(F,G);
      DMPZdtor(G);
      DMPZdtor(F);

      RingElem ans(P);
      ConvertDMPZToRingElem(ans, H);
      return ans;
    }

    CoCoA_ERROR(ERR::SERIOUS, "GCD_DMPZ");
    return f; // just to keep compiler quiet
  }


  factorization<RingElem> factor(ConstRefRingElem f)
  {
    ring R = owner(f);
    if (IsField(R) || !IsTrueGCDDomain(R))
      CoCoA_ERROR("factorization not supported", "factor(RingElem)");
    if (IsZero(f))
      CoCoA_ERROR(ERR::NotNonZero, "factor(x)");
    if (IsZZ(R))
    {
      // Factorization is actually over ZZ, so use factor(BigInt) to do the work!
      BigInt N;
      IsInteger(N, f); // guaranteed to succeed
      const factorization<BigInt> Nfacs = factor(N);
      const vector<BigInt>& prime = Nfacs.myFactors;
      vector<RingElem> facs;
      for (size_t i=0; i < prime.size(); ++i)
        facs.push_back(RingElem(R, prime[i]));
      return factorization<RingElem>(facs, Nfacs.myExponents, RingElem(R, Nfacs.myRemainingFactor));
    }
    if (!IsPolyRing(R))
      CoCoA_ERROR(ERR::NotPolyRing, "factor(RingElem)");
/////    if (!IsSparsePolyRing(R))
/////      CoCoA_ERROR(ERR::NYI, "factor(RingElem)  NYI over DUP");
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    if (IsQQ(CoeffRing(P)))
    {
      // case P = QQ[x,y,z,...]
      FractionField Q = AsFractionField(CoeffRing(P));
      RingHom coeff = CoeffEmbeddingHom(P)(EmbeddingHom(Q));
      RingElem D = CommonDenom(f);
      NVARS = NumIndets(P); // NVARS is a global!!!
      DMPZ g = ConvertToDMPZ(f*coeff(D));
      factorization<RingElem> ans = ConvertToFactorList(DMPZfactor(g), P); // BUG BUG BUG leaks the factorlist
      DMPZdtor(g);
      if (!IsOne(D)) ans.myRemainingFactor /= CanonicalHom(RingZZ(), P)(D);
      return ans;
    }
    if (IsZZ(CoeffRing(P)))
    {
      // We do not attempt to factorize the content in ZZ[x,y,z] as it could easily be very costly.
      PolyRing Qx = NewPolyRing(RingQQ(), NumIndets(P));
      RingHom phi = PolyRingHom(P, Qx, ZZEmbeddingHom(RingQQ()), indets(Qx)); // maps ZZ[x,y,z] --> QQ[x,y,z]
      RingElem FinQx = phi(f);
      factorization<RingElem> FactorizationInQx = factor(FinQx);
      const vector<RingElem>& FacsInQx = FactorizationInQx.myFactors;
      vector<RingElem> FacsInP;
      const RingElem ContentInP = CoeffEmbeddingHom(P)(num(FactorizationInQx.myRemainingFactor));
      RingHom psi = PolyRingHom(Qx, P, QQEmbeddingHom(RingZZ()), indets(P)); // maps QQ[x,y,z] --> ZZ[x,y,z]
      for (size_t i=0; i < FacsInQx.size(); ++i)
      {
        FacsInP.push_back(psi(FacsInQx[i]));
      }
      return factorization<RingElem>(FacsInP, FactorizationInQx.myExponents, ContentInP);
    }
    if (IsFiniteField(CoeffRing(P)))
    {
      // Case FF(q)[x,y,z,...]
      ring FFq = CoeffRing(P);
      if (LogCardinality(FFq) > 1)
        CoCoA_ERROR(ERR::NYI, "Factorization over alg extn of finite field");
      if (NumIndets(P) == 1)
      {
        const RingElem x = indet(P, 0);
        DUPFF F = ConvertToDUPFF(f);
        DUPFFlist facs = DUPFFfactor(F);
        DUPFFfree(F);
        const long n = DUPFFlist_length(facs);
        vector<RingElem> irreds; irreds.reserve(n);
        vector<long> multiplicity; multiplicity.reserve(n);
        for (DUPFFlist curr=facs; curr != 0/*nullptr*/; curr = curr->next)
        {
          irreds.push_back(monic(ConvertDUPFFToRingElem(curr->poly, x)));
          multiplicity.push_back(curr->deg);
        }
        DUPFFlist_dtor(facs);
        return factorization<RingElem>(irreds, multiplicity, CoeffEmbeddingHom(P)(LC(f)));
      }
      CoCoA_ERROR(ERR::NYI, "Multivariate factorization over finite field");
    }
    CoCoA_ERROR(ERR::NYI, "Factorization not in QQ[x,y,...]  or in ZZ/(p)[x,y,...]");
    return factorization<RingElem>(vector<RingElem>(),vector<long>(),RingElem(P)); // NEVER EXECUTED, just to keep compiler quiet
  }


  // temporary: by AMB 2008-01-25
  bool IsIrredPoly(ConstRefRingElem f)
  { 
    if (!IsPolyRing(owner(f)))
      CoCoA_ERROR(ERR::NotPolyRing, "IsIrredPoly");
 // ??? warning: factorize syntax might change: now only for multivariate
    if (IsConstant(f)) return IsIrred(LC(f));
    // ANNA: should do it for Z as well (taking care of constant factors)
    PolyRing P(AsPolyRing(owner(f)));
    if (!IsField(CoeffRing(P)))
      CoCoA_ERROR(ERR::NYI, "IsIrredPoly, !IsField(CoeffRing)");
    if ( IsSparsePolyRing(P) )
    {
      factorization<RingElem> facs = factor(f);
      return (IsInvertible(facs.myRemainingFactor) && facs.myFactors.size() == 1 && facs.myExponents[0] == 1);
    }
    else
    {
      PolyRing SPR = NewPolyRing(CoeffRing(P), NumIndets(P));
      RingHom phi(PolyAlgebraHom(P, SPR, indets(SPR)));
      factorization<RingElem> facs = factor(phi(f));
      return (IsInvertible(facs.myRemainingFactor) && facs.myFactors.size() == 1 && facs.myExponents[0] == 1);
    }

  }


  factorization<RingElem> SqfrDecomp_generic(ConstRefRingElem x)
  {
    factorization<RingElem> facs = factor(x);
    return facs;  /// HACK HACK HACK just so I can save the file
//     long MaxExp = 0;
//     for (long i=0; i < len(facs.myExponents); ++i)
//       MaxExp = max(MaxExp, facs.myExponents[i]);

//     const ring R = owner(x);
//     vector<RingElem> ftable(MaxExp, one(R));
//     for (long i=0; i < len(facs.myExponents); ++i)
//     {
//       const long n = facs.myExponents[i];
//       ftable[n] *= facs.myFactors[i];
//     }

//     return factorization(f,);
  }

  factorization<RingElem> SqfreeFactor(ConstRefRingElem x)
  {
    const char* const FnName = "SqfreeFactor";
    const ring xRing = owner(x);
    if (!IsPolyRing(xRing))
      CoCoA_ERROR(ERR::NYI, FnName);
    if (!IsTrueGCDDomain(xRing))
      CoCoA_ERROR(ERR::NotTrueGCDDomain, FnName);
    if (IsZero(x))
      CoCoA_ERROR(ERR::NotNonZero, FnName);
    const PolyRing Rx = AsPolyRing(xRing);

    const ring R = CoeffRing(Rx);
    return SqfrDecomp_generic(x); /// HACK HACK HACK just so I can save the file
  }


  void ContentFreeFactorLoop(vector<RingElem>& ans, const RingElem& f, vector<bool> skip)
  {
    CoCoA_ASSERT(!IsZero(f));
    CoCoA_ASSERT(IsSparsePolyRing(owner(f)));
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    CoCoA_ASSERT(IsTrueGCDDomain(CoeffRing(P)) || IsField(CoeffRing(P))); // coeffs are in field or GCD domain
    const vector<long> var = indets(f);
    if (var.empty()) return;
    for (int i=0; i < len(var); ++i)
    {
      if (skip[var[i]]) continue;
      const RingElem c = ContentWRT(f, indet(P,var[i]));
      if (IsOne(c)) continue;
      f /= c;
      skip[i] = true;
      ContentFreeFactorLoop(ans, c, skip);
    }
    if (StdDeg(f) == 0) return;
    // f is non-trivial; normalize it before adjoining it to the result.
    if (IsField(CoeffRing(P)))
    {
      ans.push_back(monic(f));
      return;
    }
    // otherwise IsTrueGCDDomain(CoeffRing(P))
    {
      RingHom embed = CoeffEmbeddingHom(P);
      ans.push_back(f/embed(content(f)));
      return;
    }
  }

  factorization<RingElem> ContentFreeFactor(ConstRefRingElem f)
  {
    const char* const FnName = "ContentFreeFactor";
    if (!IsPolyRing(owner(f)))
      CoCoA_ERROR(ERR::NotPolyRing, FnName);
    SparsePolyRing P = AsSparsePolyRing(owner(f));
    if (!IsTrueGCDDomain(CoeffRing(P)))
      CoCoA_ERROR(ERR::NotTrueGCDDomain, FnName);
    if (IsZero(f))
      CoCoA_ERROR(ERR::NotNonZero, FnName);
    vector<RingElem> facs;
    ContentFreeFactorLoop(facs, f, vector<bool>(NumIndets(P)));
    vector<long> exps(len(facs), 1);
    RingElem RemainingFactor = LC(f);
    for (int i=0; i < len(facs); ++i)
      RemainingFactor /= LC(facs[i]);
    return factorization<RingElem>(facs, exps, CoeffEmbeddingHom(P)(RemainingFactor));
  }


} // end of namespace CoCoA

// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/TmpFactor.C,v 1.21 2012/05/28 09:18:20 abbott Exp $
// $Log: TmpFactor.C,v $
// Revision 1.21  2012/05/28 09:18:20  abbott
// Created IntOperations which gathers together all operations on
// integers (both big and small).  Many consequential changes.
//
// Revision 1.20  2012/05/22 10:02:37  abbott
// Removed IsGCDDomain; substituted by IsTrueGCDDomain.
// Added IsFractionFieldOfGCDDomain.
//
// Revision 1.19  2012/05/04 20:02:20  abbott
// Added brackets as suggested by newish version of g++.
//
// Revision 1.18  2012/04/27 15:10:00  abbott
// Cleaned ConvertDUPFFToRingElem; inproved factor fn.
//
// Revision 1.17  2012/04/18 14:26:29  abbott
// Corrected an incorrect assertion (found via anna.cocoa5 test).
//
// Revision 1.16  2012/04/15 20:14:33  abbott
// Added several fns:
//   temporary hacks GCD_DUPFF and GCD_DMPZ to use the old CoCoA-4 code
//   ContentFreeFactor.
//
// Revision 1.15  2012/02/24 13:47:29  abbott
// Temporary check-in of a version which compiles (but new fns DO NOT YET WORK).
//
// Revision 1.14  2012/02/10 10:29:07  bigatti
// -- changed RingZ.H, RingQ.H --> RingZZ.H, RingQQ.H
//
// Revision 1.13  2012/02/08 15:14:30  bigatti
// -- changed: Z,Q -> ZZ,QQ
//
// Revision 1.12  2011/08/14 15:52:16  abbott
// Changed ZZ into BigInt (phase 1: just the library sources).
//
// Revision 1.11  2011/06/27 12:56:37  bigatti
// -- just a comment Anna --> AMB
//
// Revision 1.10  2009/12/03 17:41:20  abbott
// Added include of cstdlib for malloc.
//
// Revision 1.9  2009/10/26 15:43:25  bigatti
// -- added CopyFromMPZ in ZZ ctor
//
// Revision 1.8  2009/07/24 14:22:40  abbott
// Changed interface to factorizer: new function name (now "factor") with new signature.
// Did some cleaning too.
//
// Revision 1.7  2009/07/02 16:34:32  abbott
// Minor change to keep the compiler quiet.
//
// Revision 1.6  2009/05/14 09:20:03  abbott
// Changed a comment.
//
// Revision 1.5  2008/10/07 15:46:10  abbott
// Added missing log/history lines at the ends of a few files.
//
//
